%----------------------------------------------------------------------------------
% Description :
% This function M-file generates continuous impulse responses on the basis of TSV model
%
% Programmed by R. Funada, H. Harada, H. Sawada, Y. Shoji, and S. Kato (18-Dec-2006)
% Program tips
%	Tab size: 4
%	Indent size:4
%----------------------------------------------------------------------------------

function [beta,h,aoa,t,t0,np,cl_idx] = tg3c_tsv_ct_pre_fin_rev4_demo(...
	nlos, num_channels,...								% Channel params
	adist, fc, los_beta_flg, L_pl,...					% T-S-V model params
	Lam, lambda, Gam, gamma, std_ln_1, std_ln_2, ...	% SV model params
	Lmean, Omega0, smallk, sigma_fai,...
	tx_hpang, rx_hpang, ant_type)						% Antenna gain params

% Arguments:
% nlos				: Flag of NLOS environment
% num_channels		: Number of channel realizations
% Lam				: Cluster arrival rate (clusters per nsec)
% lambda			: Ray arrival rate (rays per nsec)
% Gam				: Cluster decay factor (time constant, nsec)
% gamma				: Ray decay factor (time constant, nsec)
% std_ln_1			: Standard deviation of log-normal variable for cluster fading
% std_ln_2			: Standard deviation of log-normal variable for ray fading
% Lmean				: Number of Average arrival clusters

% New paraemters added by NICT
% fc				: Carrier frequency [Hz]
% los_beta_flg	: Flag used for beta calculation
% L_pl				: path loss regarding LOS component
% Omega0			: Cluster power level
% smallk			: Small Rician effect
% sigma_fai			: Cluster arrival angle spread in deg
% tx_hpang			: TX half-power angle in deg
% rx_hpang			: RX half-power angle in deg
% ant_type			: Antenna pattern
%	1: Simple Gaussian distribution

% Output values:
%	h		: Continuous impulse responses of SV clusters
%	t_ct	: Time of arrival of h_ct
%	t0		: Arrival time of the first ray of the first SV cluster
%	np		: Number of paths of SV clusters
% New output values
%	beta	: Impulse response of the LOS component
%	aoa		: Angle of arrival of each impulse response of SV clusters

%****************** Initialize and precompute some things ******************
std_L	= 1/sqrt(2*Lam);	% std dev (nsec) of cluster arrival spacing
std_lam = 1/sqrt(2*lambda); % std dev (nsec) of ray arrival spacing

%************************** Simulation preparation *************************
h_len = 1000;  % there must be a better estimate of # of paths than this
ngrow = 1000; % amount to grow data structure if more paths are needed

%Output variables
beta	= zeros(1,num_channels);
h	= zeros(h_len,num_channels);	%renamed by NICT
aoa	= zeros(h_len,num_channels);	%added by NICT
cl_idx	= zeros(h_len,num_channels);	%for demo added by NICT

t		= zeros(h_len,num_channels);
t0		= zeros(1,num_channels);
np		= zeros(1,num_channels);

%***************************** Simulation start ****************************
for k = 1:num_channels	   % loop over number of channels

	tmp_h = zeros(size(h,1),1);
	tmp_t = zeros(size(h,1),1);
	tmp_aoa = zeros(size(h,1),1);	%added by NICT
	tmp_clidx = zeros(size(h,1),1);	%added by NICT

	%Set the number of generated clusters
	%L is the total number of clusters (TBD, see pps 9 and 10 in IEEE 15-06-0195-03-003c)
	%L = max(1, poissrnd(Lmean));       % number of clusters
	L = Lmean;							%added by NICT

	%Initialize counter regarding the number of rays in SV clusters
	path_ix = 0;

	%************************ LOS component calculation ************************

	%************************ LOS component calculation ************************

	%The following lines are added by NICT
	if nlos==0	% LOS condition based on TSV model
		if 	los_beta_flg == 1 % Desktop model
			% Compute LOS component on the basis of TSV model
			[beta0] = tg3c_tsv_beta_calc_pre_fin_rev3(fc, adist, tx_hpang, rx_hpang, ant_type);
			% beta0 is multiplied by LOS path loss
			beta(k)=beta0;			
		else  % The other LOS models
			% LOS path loss
			beta(k)=1;
		end	
		path_ix = path_ix + 1; %path_ix=1;
		tmp_h(path_ix)=beta(k);
		tmp_t(path_ix) = 0;
		tmp_clidx(path_ix) = 1;
		tmp_aoa(path_ix) = 0;
	else	% NLOS condition based on TSV model
		if 	los_beta_flg == -1 % LOS extraction mode
			beta(k)=0;			
		else
			beta(k)=1;
			path_ix = path_ix + 1; %path_ix=1;
			tmp_h(path_ix)=beta(k);
			tmp_t(path_ix) = 0;
			tmp_clidx(path_ix) = 1;
			tmp_aoa(path_ix) = 0;
		end
	end

	%************************** SV cluster computation *************************
	% Determine time-of-arrival and angle-of-arrival of the fisrt SV cluster
	Tc				= (std_L*randn)^2 + (std_L*randn)^2;
		
	if 	nlos == 1 && los_beta_flg == -1
		t0(k)		= Tc;
	end
	
	%added by NICT
	%Angle-of-arrival of clusters is distributed according to uniform distribution
	cl_ang_deg		= 360*rand-180;

	% delta K factor
	dK	= L_pl-Omega0;  %added by NICT
	Tc0	= Tc;
	
	for ncluster = 1:L
		% First ray arrival defined to be time 0 relative to cluster
		Tr		  = 0;

		%added by NICT
		%Flag: fray = 1 when it is the first ray in a cluster
		fray		= 1;

		Mcluster = std_ln_1*randn;
		%Pcluster = 10*log10(exp(-1*Tc/Gam))+Mcluster; % total cluster power
		
		%added by NICT
		%The first ray of the first cluster are related to delta K factor
		Pcluster = (-dK-10*(Tc-Tc0)/Gam./log(10))+Mcluster;
   
		Tr_len = 10*gamma;
		while (Tr < Tr_len),
			t_val = (Tc+Tr);  % time of arrival of this ray
			
			%--------------------------------------------------------------------------------------------------
			%The following lines are added by NICT			
			% Momo: first ray of the first cluster is set to the mean of the cluster.
			if fray == 1	% AOA = cluster arrival angle (first ray in each cluster)
				ray_ang_deg = cl_ang_deg;
			else			% AOA = cluster arrival angle + ray arrival angle
				% Recalculate if angle of arrival is more than 180 or less than -180
				while 1
					% Determine AOA of the ray employing Laplace distribution in deg
					ray_ang_deg0 = laplace_gen(sigma_fai); % average is 0 deg
					if abs(ray_ang_deg0) <= 180
						break;
					end
				end
				ray_ang_deg = cl_ang_deg+ray_ang_deg0;
			end

			aoa_val	= exp(j.*ray_ang_deg./180*pi);
			ray_ang_deg2 = angle(aoa_val)/pi*180;

			% Determine antenna horizontal gain
			tGrh	= calc_ant_gain_pre_fin_rev3(ant_type,rx_hpang, ray_ang_deg2);
			
			Mray = std_ln_2*randn;
			if fray == 1 %First ray of a cluster
				%Pray = 10*log10(exp(-Tr/gamma))+Mray;
				Pray = Mray;		%Tr = 0 if small_dk = 0
				% Set flag to be 0 after the first-ray's power calculation
				fray=0;
			else
				% Convert the base of small Racian facter
				small_dk = smallk.*10*log10(exp(1));
				Pray = -10*Tr/gamma./log(10)-small_dk+Mray;
				%Pray=10*log10(exp(-Tr/gamma))-small_dk+Mray;
			end

			pk = exp(j*2*pi*rand);
			% h_val = phase rotation x amplitude x electric strength of antenna
			h_val = pk *10^((Pcluster+Pray)/20)*tGrh;
			
			% Compute free space path loss of the ray from its time of arrival
			ramda	= 3e8/fc;
			Dt		= adist+0.3*t_val; %Travel distance of the ray in SV clusters (meter)
			% Free space path loss at Dt [m]
			Pdt		= 20*log10(ramda/(4*pi*Dt));

			%Threshold of path loss
			pl_th = 10*log10(4);
			% The following lines are performed if power of the ray are less than free space path loss + 3 dB
			if Pcluster+Pray+L_pl<(pl_th+Pdt),
				% The following lines are the same as that of 15.4a Matlab code except for some notes
				% Increment the number of paths
				path_ix = path_ix + 1;
				
				if path_ix > h_len,
					% grow the output structures to handle more paths as needed
					tmp_h = [tmp_h; zeros(ngrow,1)];
					tmp_t = [tmp_t; zeros(ngrow,1)];
					h = [h; zeros(ngrow,num_channels)];
					t = [t; zeros(ngrow,num_channels)];
					
					%Added by NICT
					tmp_aoa = [tmp_aoa; zeros(ngrow,1)];
					tmp_clidx = [tmp_clidx; zeros(ngrow,1)];
					aoa = [aoa; zeros(ngrow,num_channels)];
					cl_idx = [cl_idx; zeros(ngrow,num_channels)];

					h_len = h_len + ngrow;
				end
				tmp_h(path_ix) = h_val;
				tmp_t(path_ix) = t_val;
				tmp_clidx(path_ix) = ncluster;

				%Added by NICT
				tmp_aoa(path_ix) = aoa_val;

				Tr = Tr + (std_lam*randn)^2 + (std_lam*randn)^2;
			%else %Added by NICT
			%	disp('Recalculation') %Added by NICT
			end
		end
		% Set the time-of-arrivaltime-of-arrival and angle-of-arrival of the next cluster to be generated
		Tc = Tc + (std_L*randn)^2 + (std_L*randn)^2;
		cl_ang_deg = 360*rand-180; %Added by NICT
	end

	% The following lines are the same as that of 15.4a Matlab code except for some notes
	%********************************* Sorting *********************************
	np(k) = path_ix;									% Number of rays (or paths) for this realization
	[sort_tmp_t,sort_ix] = sort(tmp_t(1:np(k)));		% sort in ascending time order
	t(1:np(k),k) = sort_tmp_t;
	h(1:np(k),k) = tmp_h(sort_ix(1:np(k)));
	aoa(1:np(k),k) = tmp_aoa(sort_ix(1:np(k)));		%Added by NICT
	cl_idx(1:np(k),k) = tmp_clidx(sort_ix(1:np(k)));		%Added by NICT
	%Store the index of SV clusters attached to each ray
end