%----------------------------------------------------------------------------------
%
% Description :
% This function M-file generates sets of AOA, TOA, and power of rays in continuous
% time on the basis of TSV model
% IEEE802.15.4a distribution code was modified by
% H. Harada, R. Funada, H. Sawada, Y. Shoji, and S. Kato
%----------------------------------------------------------------------------------

function [beta,h,h2,aoa,t,t0,np,cl_idx] = tg3c_tsv_ct_r5(...
	nlos, num_channels,...								% Channel params
	adist, fc, los_beta_flg, L_pl,...					% T-S-V model params
	Lam, lambda, Gam, gamma, std_ln_1, std_ln_2, ...	% SV model params
	Lmean, Omega0, smallk, sigma_fai,...
	tx_hpbw, rx_hpbw, ant_type,...						% Antenna model params used in simulation
	op_num)

% Arguments:
% nlos				: Flag of NLOS environment
% num_channels		: Number of channel realizations
% Lam				: Cluster arrival rate (clusters per nsec)
% lambda			: Ray arrival rate (rays per nsec)
% Gam				: Cluster decay factor (time constant, nsec)
% gamma				: Ray decay factor (time constant, nsec)
% std_ln_1			: Standard deviation of log-normal variable for cluster fading
% std_ln_2			: Standard deviation of log-normal variable for ray fading
% Lmean				: Average number of arrival clusters

% Parameters added for making TG3c channel model
% fc				: Carrier frequency [GHz]
% los_beta_flg		: Flag used for beta calculation
% L_pl				: Path loss regarding LOS component
% Omega0			: Cluster power level
% smallk			: Small Rician effect
% sigma_fai			: Cluster arrival angle spread in deg
% tx_hpbw			: TX half-power angle in deg
% rx_hpbw			: RX half-power angle in deg
% ant_type			: Antenna model used in simulation
%	1: Simple Gaussian distribution
%	2: Reference antenna model
% op_num			: Option index

% Output values:
%	h		: Impulse responses in continuous time
%	h2		: Impulse responses after antenna gain convolution in continuous time
%	t		: TOAs of h
%	t0		: Arrival time of the first ray of the first SV cluster
%	np		: Number of paths in clusters including LOS component
% Output values added for making TG3c channel model
%	beta	: Amplitude of the LOS component
%	aoa		: AOAs of rays in clusters including LOS component in continuous time

%****************** Initialize and precompute some things ******************
std_L	= 1/sqrt(2*Lam);	% std dev (nsec) of cluster arrival spacing
std_lam = 1/sqrt(2*lambda); % std dev (nsec) of ray arrival spacing

%************************** Simulation preparation *************************
h_len = 1000;  % there must be a better estimate of # of paths than this
ngrow = 1000; % amount to grow data structure if more paths are needed

%Output variables
beta	= zeros(1,num_channels);
h	= zeros(h_len,num_channels);
t		= zeros(h_len,num_channels);
t0		= zeros(1,num_channels);
np		= zeros(1,num_channels);
aoa	= zeros(h_len,num_channels);		%added for making TG3c channel model
cl_idx	= zeros(h_len,num_channels);	%added for making TG3c channel model for display

for k = 1:num_channels	   % loop over number of channels

	tmp_h = zeros(size(h,1),1);
	tmp_t = zeros(size(h,1),1);
	tmp_aoa = zeros(size(h,1),1);		%added for making TG3c channel model
	tmp_clidx = zeros(size(h,1),1);		%added for making TG3c channel model

	%Set the number of generated clusters
	L = max(1, tsv_poissrnd(Lmean));	% tsv_poissonrnd.m is added for making TG3c channel model

	%Initialize counter regarding the number of rays in clusters including
	%LOS component
	path_ix = 0;

	%************************ LOS component calculation ************************

	%The following lines are added for making TG3c channel model
	if nlos==0	% LOS condition based on TSV model
		if 	los_beta_flg == 1 % Desktop TSV model
			% Compute LOS component on the basis of TSV model
			[beta0] = tsv_beta_calc_r4(fc, adist, tx_hpbw, rx_hpbw, ant_type);
			% beta0 is multiplied by LOS path loss
			beta(k) = beta0;
		elseif los_beta_flg == 0  % The other LOS TSV models
			% LOS path loss
			beta(k) = 1;
		else %LOS SV model
			beta(k) = 0;
		end
		
		if los_beta_flg ~= -1
			path_ix = path_ix + 1;
			tmp_h(path_ix)=beta(k);
			tmp_t(path_ix) = 0;
			tmp_clidx(path_ix) = 1; %LOS component is assumed to be one cluster in display
			tmp_aoa(path_ix) = 0;
		end
	else	% NLOS condition
		if 	los_beta_flg ~= -1
			beta(k)=1;
		else
			beta(k)=0;  % LOS extraction mode
		end
	end

	%************************** SV cluster computation *************************
	if 	nlos == 0 && los_beta_flg == -1 %LOS SV engine
		Tc				= 0;
		cl_ang_deg		= 0;
	else
		% Determine TOA and AOA of the first SV cluster
		Tc				= (std_L*randn)^2 + (std_L*randn)^2;
		%added for making TG3c channel model
		%AOA of clusters is distributed according to the uniform distribution
		cl_ang_deg		= 360*rand-180;
	end
	
	if 	nlos == 1 && los_beta_flg == -1
		t0(k)		= Tc;
	end
	
	% delta K factor
	dK	= L_pl-Omega0;  %added for making TG3c channel model
	Tc0	= Tc;
	
	for ncluster = 1:L
		% relative arrival time of the first ray is set to be 0 in each cluster
		Tr		  = 0;

		%added for making TG3c channel model
		%fray: flag set to be 1 when it is the first arrival ray 
		fray		= 1;

		Mcluster = std_ln_1*randn;
		%Pcluster = 10*log10(exp(-1*Tc/Gam))+Mcluster; % total cluster power
		
		%added for making TG3c channel model
		%The first ray of the first cluster is related to delta K factor
		Pcluster = (-dK-10*(Tc-Tc0)/Gam./log(10))+Mcluster;
   
		Tr_len = 10*gamma;
		while (Tr < Tr_len),
			t_val = (Tc+Tr);  % TOA of this ray
			
			%--------------------------------------------------------------------------------------------------
			%The following lines are added for making TG3c channel model			
			% Memo: first ray of the first cluster is set to the mean of the cluster.
			if fray == 1	% AOA = cluster arrival angle (first ray in each cluster)
				ray_ang_deg = cl_ang_deg;
			else			% AOA = cluster arrival angle + ray arrival angle
				% Recalculate if AOA is more than 180 deg or less than -180 deg
				while 1
					% Determine AOA of the ray according to the Laplace distribution in deg
					ray_ang_deg0 = tsv_laplacernd(sigma_fai); % average is 0 deg
					if abs(ray_ang_deg0) <= 180
						break;
					end
				end
				ray_ang_deg = cl_ang_deg+ray_ang_deg0;
			end

			ray_aoa_c	= exp(j.*ray_ang_deg./180*pi);
			aoa_val = angle(ray_aoa_c)/pi*180;
			
			Mray = std_ln_2*randn;
			if fray == 1 %First ray of a cluster
				%Pray = 10*log10(exp(-Tr/gamma))+Mray;
				Pray = Mray;		%Tr = 0 if small_dk = 0
				% Set flag to be 0 after the first-ray's power calculation
				fray=0;
			else
				% Convert the base of small Rician facter
				small_dk = smallk.*10*log10(exp(1));
				Pray = -10*Tr/gamma./log(10)-small_dk+Mray;
				%Pray=10*log10(exp(-Tr/gamma))-small_dk+Mray;
			end

			h_val = 10^((Pcluster+Pray)/20);
			
			% The following lines are the same as that of 15.4a Matlab code except for some notes
			% Increment the number of paths
			path_ix = path_ix + 1;

			if path_ix > h_len,
				% grow the output structures to handle more paths as needed
				tmp_h = [tmp_h; zeros(ngrow,1)];
				tmp_t = [tmp_t; zeros(ngrow,1)];
				h = [h; zeros(ngrow,num_channels)];
				t = [t; zeros(ngrow,num_channels)];

				%added for making TG3c channel model
				tmp_aoa = [tmp_aoa; zeros(ngrow,1)];
				tmp_clidx = [tmp_clidx; zeros(ngrow,1)];
				aoa = [aoa; zeros(ngrow,num_channels)];
				cl_idx = [cl_idx; zeros(ngrow,num_channels)];

				h_len = h_len + ngrow;
			end
			tmp_h(path_ix) = h_val;
			tmp_t(path_ix) = t_val;
			tmp_clidx(path_ix) = ncluster+1;

			%added for making TG3c channel model
			tmp_aoa(path_ix) = aoa_val;

			Tr = Tr + (std_lam*randn)^2 + (std_lam*randn)^2;
			
		end
		% Set the TOA and AOA of the next cluster to be generated
		Tc = Tc + (std_L*randn)^2 + (std_L*randn)^2;
		cl_ang_deg = 360*rand-180; %added for making TG3c channel model
	end

	% The following lines are the same as that of 15.4a Matlab code except for some notes
	%********************************* Sorting *********************************
	np(k) = path_ix;									% Number of rays (or paths) for this realization
	[sort_tmp_t,sort_ix] = sort(tmp_t(1:np(k)));		% sort in ascending time order
	t(1:np(k),k) = sort_tmp_t;
	h(1:np(k),k) = tmp_h(sort_ix(1:np(k)));
	aoa(1:np(k),k) = tmp_aoa(sort_ix(1:np(k)));			%added for making TG3c channel model
	%Attach the generated cluster index to each ray
	cl_idx(1:np(k),k) = tmp_clidx(sort_ix(1:np(k)));	%added for making TG3c channel model
end

%************** Generate continuous complex impulse responses **************
%**************                 with antenna gain convolution **************
% This lines are added for making TG3c channel model
if op_num == 2 || op_num == 3
	tGrh	= tsv_ant_gain_r5(ant_type,rx_hpbw, aoa);
	for ij=1:num_channels
		tGrh(np(ij)+1:end,ij)=0;
	end
	h2 = h.*tGrh;
else
	h2 = [];
end
